function [T, r]= LUT(imPts, marksize, xg)
% function [T, r]= LUT(imPts, marksize, xg)

arc= pi/180;
d= sqrt(2)*marksize/2;

%% 1 calculate mark's center =============================================
m0= intersect_line_line(imPts(:,1),imPts(:,3)-imPts(:,1),...
    imPts(:,2),imPts(:,4)-imPts(:,2));
m0x= m0(1);
m0y= m0(2);

%% 2 find the biggest theta_i
% a^2= b^2+c^2-2b*c*cos(theta)
% 2*cos(ti)*l0= (l0^2+li^2-di^2)/li
% 4*cos(ti)^2*l0^2= (l0^2+li^2-di^2)^2/li^2
li_2= imPts(1,:).^2+imPts(2,:).^2+1;                 % li^2
di_2= (imPts(1,:)-m0x).^2+(imPts(2,:)-m0y).^2;       % di^2
l0_2= m0x^2+m0y^2+1;                                 % l0^2
cosai= (l0_2+li_2-di_2).^2./li_2;
[para iFirstImPt]= min(cosai);

%% 3 define m1,m2,m3,m4 ==================================================
if iFirstImPt == 1
    mi= imPts(:,[1 3 2 4]);
elseif iFirstImPt == 2
    mi= imPts(:,[2 4 3 1]);
elseif iFirstImPt == 3
    mi= imPts(:,[3 1 4 2]);
elseif iFirstImPt == 4
    mi= imPts(:,[4 2 1 3]);
end

%% 4 cal new cam matrix and transform image points =======================
axisYprox= [mi(:,1)-m0; 0];     % let longest edge along the y axis
axisZ= [m0x; m0y; 1];
axisX= cross2(axisYprox,axisZ);
axisY= cross2(axisZ,axisX);

axisX= axisX/norm(axisX);
axisY= axisY/norm(axisY);
axisZ= axisZ/norm(axisZ);

Rp= [axisX axisY axisZ];      % cam-newcam-R
mi= Rp.'*[mi; 1 1 1 1];
mi= [mi(1,:)./mi(3,:);mi(2,:)./mi(3,:)];

%% 5 measure the theta= theta1+theta2 ====================================
theta= atan(norm(mi(:,1)))+atan(norm(mi(:,2)));
theta= theta/arc;

%% 6 select the table ====================================================
iTheta= (theta-xg.theta_a)/xg.dThetas +1;
if iTheta < 1
    iTheta= 1;
end
if iTheta > xg.nThetas
    iTheta= xg.nThetas;
end

%% 7 beta ================================================================
% beta
beta3= lut3d(iTheta,mi(1,3),mi(2,3),xg);
beta4= lut3d(iTheta,mi(1,4),mi(2,4),xg);
beta= -(beta3+beta4)/2;
beta= beta*arc;

% inv beta (ambiguous poses)
beta3= lut3d(iTheta,mi(1,3),-mi(2,3),xg);
beta4= lut3d(iTheta,mi(1,4),-mi(2,4),xg);
invbeta= (beta3+beta4)/2;
invbeta= invbeta*arc;
        
%% 8 t0 ==================================================================
t0= caltzfrombeta(theta, beta/arc, xg);
invt0= caltzfrombeta(theta, invbeta/arc, xg);

%% 9 gamma ===============================================================
if abs(beta) > 2*arc
    gamma= atan((mi(2,3)-mi(2,4))/(mi(1,3)-mi(1,4))/sin(beta));
    invgamma= atan((mi(2,3)-mi(2,4))/(mi(1,3)-mi(1,4))/sin(invbeta));
else
    theta34= atan(norm(mi(:,3)))+atan(norm(mi(:,4)));
    cosgamma= tan(theta34)*(t0^2-xg.b^2)/(2*xg.b*t0);
    if cosgamma > 1, cosgamma= 1; end
    gamma= acos(cosgamma);
    invgamma= -gamma;
end

if mi(1,3) > 0
    % two case: gamma or gamma+180
    gamma= gamma+pi;
    invgamma= invgamma+pi;
end

% for different marksize
t0= t0/xg.b*d;
invt0= invt0/xg.b*d;

%% ti ====================================================================
ti(1)= sqrt(d^2*cos(beta)^2+(t0+d*sin(beta))^2);
ti(2)= sqrt(d^2*cos(beta)^2+(t0-d*sin(beta))^2);
ti(3)= sqrt(d^2*cos(gamma)^2+d^2*sin(beta)^2*sin(gamma)^2+...
    (t0+d*cos(beta)*sin(gamma))^2);
ti(4)= sqrt(d^2*cos(gamma)^2+d^2*sin(beta)^2*sin(gamma)^2+...
    (t0-d*cos(beta)*sin(gamma))^2);

invti(1)= sqrt(d^2*cos(invbeta)^2+(invt0+d*sin(invbeta))^2);
invti(2)= sqrt(d^2*cos(invbeta)^2+(invt0-d*sin(invbeta))^2);
invti(3)= sqrt(d^2*cos(invgamma)^2+d^2*sin(invbeta)^2*sin(invgamma)^2+...
    (invt0+d*cos(invbeta)*sin(invgamma))^2);
invti(4)= sqrt(d^2*cos(invgamma)^2+d^2*sin(invbeta)^2*sin(invgamma)^2+...
    (invt0-d*cos(invbeta)*sin(invgamma))^2);

if iFirstImPt == 1
    ti= ti([1 3 2 4]);
    invti= invti([1 3 2 4]);
elseif iFirstImPt == 2
    ti= ti([4 1 3 2]);
    invti= invti([4 1 3 2]);
elseif iFirstImPt == 3
    ti= ti([2 4 1 3]);
    invti= invti([2 4 1 3]);
elseif iFirstImPt == 4
    ti= ti([3 2 4 1]);
    invti= invti([3 2 4 1]);
end

% get pose
Pts= [d 0 0 1;0 d 0 1;-d 0 0 1;0 -d 0 1].';
[R,t]= calcampose(imPts, Pts(1:3,:), ti);
[iR,it]= calcampose(imPts, Pts(1:3,:), invti);

% back project
[prjpts,r]= projtarget(R,t,Pts,imPts);
[prjpts,ir]= projtarget(iR,it,Pts,imPts);

%% output the result =====================================================
if r < ir
    T= [R t];
else
    T= [iR it];
    r= ir;
end

return

% ============================================
function [R2,t2] = calcampose(xx,XXw,t)

n= 4;

X= XXw;
Y= [];
for i= 1:n
    XX= [xx(:,i); 1];
    XX= XX/norm(XX)*t(i);
    Y= [Y XX];
end

K= eye(4)-ones(4,4)/4;

ux= mean(X,2);
uy= mean(Y,2);
sigmx2= mean(sum((X*K).^2));
sigmy2= mean(sum((Y*K).^2));

SXY= Y*K*(X')/4;
[U, D, V]= svd(SXY);
S= eye(3);
if det(SXY) < 0
    S(3,3)= -1;
end

R2= U*S*(V');
c2= trace(D*S)/sigmx2;
t2= uy-c2*R2*ux;

X= R2(:,1);
Y= R2(:,2);
Z= R2(:,3);
if norm(cross(X,Y)-Z) > 2e-2
    R2(:,3)= -Z;
end

return

% ============================================
function beta = lut3d(idx,mx,my,xg)

nThetas= xg.nThetas;

% check: if idx in range of 1 to nThetas
if idx < 1
    idx= 1;
end
if idx > nThetas
    idx= nThetas;
end

C= idx;
C0= floor(C);
C1= ceil(C);

% lookup scale factor, linear interp
k= C- C0;
v2i= (1-k)*xg.v2iTable(:,C0)+ k*xg.v2iTable(:,C1);

A= my*v2i(2);
A= A+xg.tableh+1;   % 0 -> tableh+1

B= abs(mx*v2i(1));
B= B+1;          % 0 -> 1

% check range: xy 
if A >= xg.tableh*2 | A < 1
    beta = 45;
    return;
end

if B >= xg.tablew
    beta = 45;
    return;
end

% lookup table
A0= floor(A);
B0= floor(B);
A1= ceil(A);
B1= ceil(B);

T= xg.betaTable([A0 A1],[B0 B1],[C0 C1]);
beta= kinterp(T, [A-A0, B-B0, C-C0]);

if xg.bytes == 1
    beta= double(beta)/255*45;
end

return

% ============================================
function y= kinterp(A,k)

A= double(A);

A= (1-k(3))*A(:,:,1)+k(3)*A(:,:,2);
A= (1-k(2))*A(:,1)+k(2)*A(:,2);
y= A(1)+k(1)*(A(2)-A(1));

return

% ============================================
function t0= caltzfrombeta(alpha, beta, xg)

b= xg.b;
arc= pi/180;

alpha= alpha*arc;
beta= beta*arc;

cosbeta= cos(beta);
tanalpha= tan(alpha);
t0= b*(cosbeta+sqrt(cosbeta^2+tanalpha^2))/tanalpha;

return

% ============================================
function xpt = intersect_line_line(p1,v1,p2,v2)

S= v1(1)*v2(2)-v1(2)*v2(1);
if abs(S) < 20*eps
    disp('error image points!');    
    return
end

dp= p1-p2;
t= (dp(2)*v2(1)-dp(1)*v2(2))/S;

xpt= p1+v1*t;

return