function [resdata, restime]= random_test(randdata,noiselevel,...
    focal,targetsize,tablename)

restime= [];
resdata= {};
warning off all;

% init the lut
xg= initlut(tablename);

% add noise to image points
ndata= length(randdata);
for i= 1:ndata
    % normalized coordinates
    impts_noise{i}= randdata{i}{2} + randn(2,4)*(noiselevel/focal);
end

% 1000 random test

% HOMO --------------------------------------------------
tmp= zeros(ndata,2);
tic;
for i= 1:ndata
    
    impts= impts_noise{i};
    Ttrue= randdata{i}{1};

    T= HOMO(impts, targetsize);
    
    y= cal_pose_err(T,Ttrue);
    tmp(i,:)= [y(1) y(2)];

    showpercent(i,ndata);
end
restime(end+1)= toc;
resdata{end+1}= tmp;
fprintf('\b\b\b\bHomo\t');
fprintf('%.2f\t%.2f\t%d\n',mean(tmp),sum(tmp(:,1) < 15));

% LUT --------------------------------------------------
tmp= zeros(ndata,2);
tic;
for i= 1:ndata
    
    impts= impts_noise{i};
    Ttrue= randdata{i}{1};

    T= LUT(impts, targetsize, xg);
    
    y= cal_pose_err(T,Ttrue);
    tmp(i,:)= [y(1) y(2)];

    showpercent(i,ndata);
end
restime(end+1)= toc;
resdata{end+1}= tmp;
fprintf('\b\b\b\bLUT \t');
fprintf('%.2f\t%.2f\t%d\n',mean(tmp),sum(tmp(:,1) < 15));

% LMF ---------------------------------------------------
tmp= zeros(ndata,2);
tic;
for i= 1:ndata
    
    impts= impts_noise{i};
    Ttrue= randdata{i}{1};

    T= LMF(impts, targetsize);
    
    y= cal_pose_err(T,Ttrue);
    tmp(i,:)= [y(1) y(2)];

    showpercent(i,ndata);
end
restime(end+1)= toc;
resdata{end+1}= tmp;
fprintf('\b\b\b\bLMF \t');
fprintf('%.2f\t%.2f\t%d\n',mean(tmp),sum(tmp(:,1) < 15));

% LHM --------------------------------------------------
tmp= zeros(ndata,2);
tic;
for i= 1:ndata
    
    impts= impts_noise{i};
    Ttrue= randdata{i}{1};

    T= LHM(impts, targetsize);
    
    y= cal_pose_err(T,Ttrue);
    tmp(i,:)= [y(1) y(2)];

    showpercent(i,ndata);
end
restime(end+1)= toc;
resdata{end+1}= tmp;
fprintf('\b\b\b\bLHM \t');
fprintf('%.2f\t%.2f\t%d\n',mean(tmp),sum(tmp(:,1) < 15));

% SP ---------------------------------------------------
tmp= zeros(ndata,2);
tic;
for i= 1:ndata
    
    impts= impts_noise{i};
    Ttrue= randdata{i}{1};

    T= SP(impts, targetsize);
    
    y= cal_pose_err(T,Ttrue);
    tmp(i,:)= [y(1) y(2)];

    showpercent(i,ndata);
end
restime(end+1)= toc;
resdata{end+1}= tmp;
fprintf('\b\b\b\bSP  \t');
fprintf('%.2f\t%.2f\t%d\n',mean(tmp),sum(tmp(:,1) < 15));

% the average time
restime= restime/ndata;

return
