function [R, T]=getrotT(wpts,cpts);
  
% Xu Chi, Huazhong University of Science and Technology, China
% 14-Aug-2009

n=size(wpts,1);

h= ones(n,1);
K= eye(n)-h*h'/n;
X= wpts';
Y= cpts';

ux= mean(X,2);
uy= mean(Y,2);
sigmx2= mean(sum((X*K).^2));
sigmy2= mean(sum((Y*K).^2));

SXY= Y*K*(X')/n;
[U, D, V]= svd(SXY);
S= eye(3);
if det(SXY) < 0
    S(3,3)= -1;
end

R= U*S*(V');
c= trace(D*S)/sigmx2;
T= uy-c*R*ux;

X= R(:,1);
Y= R(:,2);
Z= R(:,3);
if norm(cross(X,Y)-Z) > 2e-2
    R(:,3)= -Z;
end

return

% This routine solves the exterior orientation problem for a point cloud
%  given in both camera and world coordinates. 
  
% wpts = 3D points in arbitrary reference frame
% cpts = 3D points in camera reference frame
  
n=size(wpts,1);
M=zeros(3);

ccent=mean(cpts);
wcent=mean(wpts);

for i=1:3
  cpts(:,i)=cpts(:,i)-ccent(i)*ones(n,1);
  wpts(:,i)=wpts(:,i)-wcent(i)*ones(n,1);
end

for i=1:n
   M=M+cpts(i,:)'*wpts(i,:);
end

[U S V]=svd(M);
R=U*V';
if det(R)<0
  R=-R;
end

T=ccent'-R*wcent';

